@@ -20,6 +20,8 @@
 
 #include "document_glom.h"
 #include "../data_structure/glomconversions.h"
+#include "../data_structure/layout/report_parts/layoutitem_summary.h"
+#include "../data_structure/layout/report_parts/layoutitem_fieldsummary.h"
 //#include "config.h" //To get GLOM_DTD_INSTALL_DIR - dependent on configure prefix.
 #include <algorithm> //For std::find_if().
 #include <sstream> //For stringstream
@@ -39,8 +41,11 @@
 
 #define GLOM_NODE_DATA_LAYOUT_PORTAL "data_layout_portal"
 #define GLOM_NODE_DATA_LAYOUT_ITEM "data_layout_item"
+#define GLOM_ATTRIBUTE_DATA_LAYOUT_ITEM_FIELD_USE_DEFAULT_FORMATTING "use_default_formatting"
 #define GLOM_NODE_DATA_LAYOUT_ITEM_GROUPBY "data_layout_item_groupby"
+#define GLOM_NODE_DATA_LAYOUT_GROUP_SECONDARYFIELDS "secondary_fields"
 #define GLOM_NODE_DATA_LAYOUT_ITEM_SUMMARY "data_layout_item_summary"
+#define GLOM_NODE_DATA_LAYOUT_ITEM_FIELDSUMMARY "data_layout_item_fieldsummary"
 #define GLOM_NODE_TABLE "table"
 #define GLOM_NODE_FIELDS "fields"
 #define GLOM_NODE_FIELD "field"
@@ -78,22 +83,7 @@
 #define GLOM_ATTRIBUTE_FIELD "field"
 #define GLOM_ATTRIBUTE_EDITABLE "editable"
 
-#define GLOM_ATTRIBUTE_FORMAT_THOUSANDS_SEPARATOR "format_thousands_separator"
-#define GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES_RESTRICTED "format_decimal_places_restricted"
-#define GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES "format_decimal_places"
-#define GLOM_ATTRIBUTE_FORMAT_CURRENCY_SYMBOL "format_currency_symbol"
 
-#define GLOM_ATTRIBUTE_FORMAT_TEXT_MULTILINE "format_text_multiline"
-
-#define GLOM_ATTRIBUTE_CHOICES_RESTRICTED "choices_restricted"
-#define GLOM_ATTRIBUTE_CHOICES_CUSTOM "choices_custom"
-#define GLOM_ATTRIBUTE_CHOICES_CUSTOM_LIST "custom_choice_list"
-#define GLOM_NODE_CUSTOM_CHOICE "custom_choice"
-#define GLOM_ATTRIBUTE_VALUE "value"
-#define GLOM_ATTRIBUTE_CHOICES_RELATED "choices_related"
-#define GLOM_ATTRIBUTE_CHOICES_RELATED_RELATIONSHIP "choices_related_relationship"
-#define GLOM_ATTRIBUTE_CHOICES_RELATED_FIELD "choices_related_field"
-#define GLOM_ATTRIBUTE_CHOICES_RELATED_SECOND "choices_related_second"
 
 #define GLOM_ATTRIBUTE_RELATIONSHIP_NAME "relationship"
 
@@ -101,9 +91,29 @@
 #define GLOM_NODE_REPORT "report"
 #define GLOM_ATTRIBUTE_REPORT_ITEM_GROUPBY_GROUPBY "groupby"
 #define GLOM_ATTRIBUTE_REPORT_ITEM_GROUPBY_SORTBY "sortby"
+#define GLOM_ATTRIBUTE_LAYOUT_ITEM_FIELDSUMMARY_SUMMARYTYPE "summarytype"
+
+#define GLOM_NODE_FORMAT "formatting"
+#define GLOM_ATTRIBUTE_FORMAT_THOUSANDS_SEPARATOR "format_thousands_separator"
+#define GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES_RESTRICTED "format_decimal_places_restricted"
+#define GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES "format_decimal_places"
+#define GLOM_ATTRIBUTE_FORMAT_CURRENCY_SYMBOL "format_currency_symbol"
+
+#define GLOM_ATTRIBUTE_FORMAT_TEXT_MULTILINE "format_text_multiline"
+
+#define GLOM_ATTRIBUTE_FORMAT_CHOICES_RESTRICTED "choices_restricted"
+#define GLOM_ATTRIBUTE_FORMAT_CHOICES_CUSTOM "choices_custom"
+#define GLOM_ATTRIBUTE_FORMAT_CHOICES_CUSTOM_LIST "custom_choice_list"
+#define GLOM_NODE_FORMAT_CUSTOM_CHOICE "custom_choice"
+#define GLOM_ATTRIBUTE_VALUE "value"
+#define GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED "choices_related"
+#define GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED_RELATIONSHIP "choices_related_relationship"
+#define GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED_FIELD "choices_related_field"
+#define GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED_SECOND "choices_related_second"
 
 Document_Glom::Document_Glom()
-: m_block_cache_update(false)
+: m_block_cache_update(false),
+  m_block_modified_set(false)
 {
   //Conscious use of virtual methods in a constructor:
   set_file_extension("glom");
@@ -283,28 +293,32 @@ void Document_Glom::remove_table(const Glib::ustring& table_name)
   for(type_tables::iterator iter = m_tables.begin(); iter != m_tables.end(); ++iter)
   {
     DocumentTableInfo& info = iter->second;
-
-    bool something_changed = true;
-    type_vecRelationships::iterator iterRel = info.m_relationships.begin();
-    while(something_changed)
+    
+    if(!(info.m_relationships.empty()))
     {
-      if(iterRel->get_to_table() == table_name)
+      type_vecRelationships::iterator iterRel = info.m_relationships.begin();
+      bool something_changed = true;
+      while(something_changed && !info.m_relationships.empty())
       {
-        //Loop again, because we have changed the structure:
-        remove_relationship(*iterRel); //Also removes anything that uses the relationship.
-
-        something_changed = true;
-        iterRel = info.m_relationships.begin();
-      }
-      else
-      {
-        if(iterRel == info.m_relationships.end())
-          something_changed = false; //We've looked at them all, without changing things.
+        if(iterRel->get_to_table() == table_name)
+        {
+          //Loop again, because we have changed the structure:
+          remove_relationship(*iterRel); //Also removes anything that uses the relationship.
+  
+          something_changed = true;
+          iterRel = info.m_relationships.begin();
+        }
+        else
+        {
+          ++iterRel;
+          
+          if(iterRel == info.m_relationships.end())
+            something_changed = false; //We've looked at them all, without changing things.
+        }
       }
     }
 
   }
-
 }
 
 
@@ -479,12 +493,18 @@ bool Document_Glom::get_node_attribute_value_as_bool(const xmlpp::Element* node,
   
 void Document_Glom::set_node_attribute_value_as_bool(xmlpp::Element* node, const Glib::ustring& strAttributeName, bool value)
 {
+  if(!value && !node->get_attribute(strAttributeName))
+    return; //Use the non-existance of an attribute to mean false, to save space.
+
   Glib::ustring strValue = (value ? "true" : "false");
   set_node_attribute_value(node, strAttributeName, strValue);
 }
 
 void Document_Glom::set_node_attribute_value_as_decimal(xmlpp::Element* node, const Glib::ustring& strAttributeName, int value)
 {
+  if(!value && !node->get_attribute(strAttributeName))
+    return; //Use the non-existance of an attribute to mean zero, to save space.
+
   //Get text representation of int:
   std::stringstream thestream;
   thestream.imbue( std::locale::classic() ); //The C locale.
@@ -854,12 +874,15 @@ Glib::ustring Document_Glom::get_default_table() const
     type_tables::const_iterator iter = m_tables.begin();
     return iter->second.m_info.m_name;
   }
-  
+
   return Glib::ustring();
 }
 
 void Document_Glom::set_modified(bool value)
 {
+  if(value && m_block_modified_set)
+    return;
+
   if(value != get_modified()) //Prevent endless loops
   {
     Bakery::Document_XML::set_modified(value);
@@ -887,10 +910,74 @@ void Document_Glom::set_modified(bool value)
   }
 }
 
+void Document_Glom::load_after_layout_item_field_formatting(const xmlpp::Element* element, FieldFormatting& format, const Field& field)
+{
+  //Numeric formatting:
+  format.m_numeric_format.m_use_thousands_separator = get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_FORMAT_THOUSANDS_SEPARATOR);
+  format.m_numeric_format.m_decimal_places_restricted = get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES_RESTRICTED);
+  format.m_numeric_format.m_decimal_places = get_node_attribute_value_as_decimal(element, GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES);
+  format.m_numeric_format.m_currency_symbol = get_node_attribute_value(element, GLOM_ATTRIBUTE_FORMAT_CURRENCY_SYMBOL);
+
+  //Text formatting:
+  format.set_text_format_multiline( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_FORMAT_TEXT_MULTILINE) );
+
+  //Choices:
+  format.set_choices_restricted( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_FORMAT_CHOICES_RESTRICTED) );
+  format.set_has_custom_choices( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_FORMAT_CHOICES_CUSTOM) );
+
+  if(format.get_has_custom_choices())
+  {
+    const xmlpp::Element* nodeChoiceList = get_node_child_named(element, GLOM_ATTRIBUTE_FORMAT_CHOICES_CUSTOM_LIST);
+    if(nodeChoiceList)
+    {
+      FieldFormatting::type_list_values list_values;
+
+      xmlpp::Node::NodeList listNodesCustomChoices = nodeChoiceList->get_children(GLOM_NODE_FORMAT_CUSTOM_CHOICE);
+      for(xmlpp::Node::NodeList::iterator iter = listNodesCustomChoices.begin(); iter != listNodesCustomChoices.end(); ++iter)
+      {
+        const xmlpp::Element* element = dynamic_cast<const xmlpp::Element*>(*iter);
+        if(element)
+        {
+          Gnome::Gda::Value value = get_node_attribute_value_as_value(element, GLOM_ATTRIBUTE_VALUE, field.get_glom_type());
+          list_values.push_back(value);
+        }
+      }
+
+      format.set_choices_custom(list_values);
+    }
+  }
+
+  format.set_has_related_choices( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED) );
+
+  format.set_choices(get_node_attribute_value(element, GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED_RELATIONSHIP),
+    get_node_attribute_value(element, GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED_FIELD),
+    get_node_attribute_value(element, GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED_SECOND) );
+  //Full details are updated in filled-in ().
+
+}
+
+void Document_Glom::load_after_layout_item_field(const xmlpp::Element* element, LayoutItem_Field& item)
+{
+  item.set_name( get_node_attribute_value(element, GLOM_ATTRIBUTE_NAME) );
+
+  item.m_relationship.set_name( get_node_attribute_value(element, GLOM_ATTRIBUTE_RELATIONSHIP_NAME) ); //Full details are updated in filled-in ().
+
+  item.set_editable( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_EDITABLE) );
+
+  const xmlpp::Element* elementFormatting = get_node_child_named(element, GLOM_NODE_FORMAT);
+  if(elementFormatting)
+    load_after_layout_item_field_formatting(elementFormatting, item.m_formatting, item.m_field);
+
+  item.set_formatting_use_default( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_DATA_LAYOUT_ITEM_FIELD_USE_DEFAULT_FORMATTING) );
+}
+
 void Document_Glom::load_after_layout_group(const xmlpp::Element* node, const Glib::ustring table_name, LayoutGroup& group)
 {
   if(!node)
+  {
+    //g_warning("Document_Glom::load_after_layout_group(): node is NULL");
     return;
+  }
 
   //Get the group details:
   group.set_name( get_node_attribute_value(node, GLOM_ATTRIBUTE_NAME) );
@@ -900,7 +987,7 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, const Gl
   group.m_sequence = get_node_attribute_value_as_decimal(node, GLOM_ATTRIBUTE_SEQUENCE);
 
   //Get the child items:
-  xmlpp::Node::NodeList listNodes = node->get_children(); 
+  xmlpp::Node::NodeList listNodes = node->get_children();
   for(xmlpp::Node::NodeList::iterator iter = listNodes.begin(); iter != listNodes.end(); ++iter)
   {
     const xmlpp::Element* element = dynamic_cast<const xmlpp::Element*>(*iter);
@@ -910,58 +997,16 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, const Gl
       if(element->get_name() == GLOM_NODE_DATA_LAYOUT_ITEM)
       {
         LayoutItem_Field item;
+        load_after_layout_item_field(element, item);
 
-        item.set_name( get_node_attribute_value(element, GLOM_ATTRIBUTE_NAME) );
-
-        item.m_relationship.set_name( get_node_attribute_value(element, GLOM_ATTRIBUTE_RELATIONSHIP_NAME) ); //Full details are updated in filled-in ().
-
-        item.set_editable( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_EDITABLE) );
-
-        //Numeric formatting:
-        NumericFormat numeric_format;
-        numeric_format.m_use_thousands_separator = get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_FORMAT_THOUSANDS_SEPARATOR);
-        numeric_format.m_decimal_places_restricted = get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES_RESTRICTED);
-        numeric_format.m_decimal_places = get_node_attribute_value_as_decimal(element, GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES);
-        numeric_format.m_currency_symbol = get_node_attribute_value(element, GLOM_ATTRIBUTE_FORMAT_CURRENCY_SYMBOL);
-
-        item.m_numeric_format = numeric_format;
-
-        //Text formatting:
-        item.set_text_format_multiline( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_FORMAT_TEXT_MULTILINE) );
-
-        //Choices:
-        item.set_choices_restricted( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_CHOICES_RESTRICTED) );
-        item.set_has_custom_choices( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_CHOICES_CUSTOM) );
-
-        if(item.get_has_custom_choices())
-        {
-          const xmlpp::Element* nodeChoiceList = get_node_child_named(element, GLOM_ATTRIBUTE_CHOICES_CUSTOM_LIST);
-          if(nodeChoiceList)
-          {
-            LayoutItem_Field::type_list_values list_values;
-
-            xmlpp::Node::NodeList listNodesCustomChoices = nodeChoiceList->get_children(GLOM_NODE_CUSTOM_CHOICE);
-            for(xmlpp::Node::NodeList::iterator iter = listNodesCustomChoices.begin(); iter != listNodesCustomChoices.end(); ++iter)
-            {
-              const xmlpp::Element* element = dynamic_cast<const xmlpp::Element*>(*iter);
-              if(element)
-              {
-                Gnome::Gda::Value value = get_node_attribute_value_as_value(element, GLOM_ATTRIBUTE_VALUE, item.m_field.get_glom_type());
-                list_values.push_back(value);
-              }
-            }
-
-            item.set_choices_custom(list_values);
-          }
-        }
-
-        item.set_has_related_choices( get_node_attribute_value_as_bool(element, GLOM_ATTRIBUTE_CHOICES_RELATED) );
-
-        item.set_choices(get_node_attribute_value(element, GLOM_ATTRIBUTE_CHOICES_RELATED_RELATIONSHIP),
-          get_node_attribute_value(element, GLOM_ATTRIBUTE_CHOICES_RELATED_FIELD),
-          get_node_attribute_value(element, GLOM_ATTRIBUTE_CHOICES_RELATED_SECOND) );
-        //Full details are updated in filled-in ().
-
+        item.m_sequence = sequence;
+        group.add_item(item, sequence);
+      }
+      else if(element->get_name() == GLOM_NODE_DATA_LAYOUT_ITEM_FIELDSUMMARY)
+      {
+        LayoutItem_FieldSummary item;
+        load_after_layout_item_field(element, item);
+        item.set_summary_type_from_sql( get_node_attribute_value(element, GLOM_ATTRIBUTE_LAYOUT_ITEM_FIELDSUMMARY_SUMMARYTYPE) );
 
         item.m_sequence = sequence;
         group.add_item(item, sequence);
@@ -996,6 +1041,25 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, const Gl
         field_sortby.set_name( get_node_attribute_value(element, GLOM_ATTRIBUTE_REPORT_ITEM_GROUPBY_SORTBY) );
         child_group.set_field_sort_by(field_sortby);
 
+        //Secondary fields:
+        xmlpp::Element* elementSecondary = get_node_child_named(element, GLOM_NODE_DATA_LAYOUT_GROUP_SECONDARYFIELDS);
+        if(elementSecondary)
+        {
+          xmlpp::Element* elementGroup = get_node_child_named(elementSecondary, GLOM_NODE_DATA_LAYOUT_GROUP);
+          if(elementGroup)
+          {
+            load_after_layout_group(elementGroup, table_name, child_group.m_group_secondary_fields);
+          }
+        }
+
+        group.add_item(child_group);
+      }
+      else if(element->get_name() == GLOM_NODE_DATA_LAYOUT_ITEM_SUMMARY)
+      {
+        LayoutItem_Summary child_group;
+        //Recurse:
+        load_after_layout_group(element, table_name, child_group);
+
         group.add_item(child_group);
       }
     }
@@ -1101,7 +1165,12 @@ bool Document_Glom::load_after()
 
                 //We set this after set_field_info(), because that gets a glom type from the (not-specified) gdatype. Yes, that's strange, and should probably be more explicit.
                 field.set_glom_type( field_type_enum );
- 
+
+                //Default Formatting:
+                const xmlpp::Element* elementFormatting = get_node_child_named(nodeChild, GLOM_NODE_FORMAT);
+                if(elementFormatting)
+                  load_after_layout_item_field_formatting(elementFormatting, field.m_default_formatting, field);
+
                 doctableinfo.m_fields.push_back(field);
               }
             }
@@ -1280,6 +1349,56 @@ bool Document_Glom::load_after()
   return result;
 }
 
+void Document_Glom::save_before_layout_item_field_formatting(xmlpp::Element* nodeItem, const FieldFormatting& format, const Field& field)
+{
+  //Numeric format:
+  set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_FORMAT_THOUSANDS_SEPARATOR,  format.m_numeric_format.m_use_thousands_separator);
+  set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES_RESTRICTED, format.m_numeric_format.m_decimal_places_restricted);
+  set_node_attribute_value_as_decimal(nodeItem, GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES, format.m_numeric_format.m_decimal_places);
+  set_node_attribute_value(nodeItem, GLOM_ATTRIBUTE_FORMAT_CURRENCY_SYMBOL, format.m_numeric_format.m_currency_symbol);
+
+  set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_FORMAT_CHOICES_RESTRICTED, format.get_choices_restricted());
+  set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_FORMAT_CHOICES_CUSTOM, format.get_has_custom_choices());
+
+  //Text formatting:
+  set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_FORMAT_TEXT_MULTILINE, format.get_text_format_multiline());
+
+  //Choices:
+  if(format.get_has_custom_choices())
+  {
+    xmlpp::Element* child = nodeItem->add_child(GLOM_ATTRIBUTE_FORMAT_CHOICES_CUSTOM_LIST);
+
+    const FieldFormatting::type_list_values list_values = format.get_choices_custom();
+    for(FieldFormatting::type_list_values::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
+    {
+      xmlpp::Element* childChoice = child->add_child(GLOM_NODE_FORMAT_CUSTOM_CHOICE);
+      set_node_attribute_value_as_value(childChoice, GLOM_ATTRIBUTE_VALUE, *iter, field.get_glom_type());
+    }
+  }
+
+  set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED, format.get_has_related_choices() );
+
+  Glib::ustring choice_relationship, choice_field, choice_second;
+  format.get_choices(choice_relationship, choice_field, choice_second);
+  set_node_attribute_value(nodeItem, GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED_RELATIONSHIP, choice_relationship);
+  set_node_attribute_value(nodeItem, GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED_FIELD, choice_field);
+  set_node_attribute_value(nodeItem, GLOM_ATTRIBUTE_FORMAT_CHOICES_RELATED_SECOND, choice_second);
+}
+
+void Document_Glom::save_before_layout_item_field(xmlpp::Element* nodeItem, const LayoutItem_Field& field)
+{
+  nodeItem->set_attribute(GLOM_ATTRIBUTE_NAME, field.get_name());
+  nodeItem->set_attribute(GLOM_ATTRIBUTE_RELATIONSHIP_NAME, field.get_relationship_name());
+  set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_EDITABLE, field.get_editable());
+
+  xmlpp::Element* elementFormat = nodeItem->add_child(GLOM_NODE_FORMAT);
+  save_before_layout_item_field_formatting(elementFormat, field.m_formatting, field.m_field);
+
+  set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_DATA_LAYOUT_ITEM_FIELD_USE_DEFAULT_FORMATTING, field.get_formatting_use_default());
+
+  set_node_attribute_value_as_decimal(nodeItem, GLOM_ATTRIBUTE_SEQUENCE, field.m_sequence);
+}
+
 void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutGroup& group)
 {
   //g_warning("save_before_layout_group");
@@ -1293,10 +1412,25 @@ void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutG
 
     set_node_attribute_value(child, GLOM_ATTRIBUTE_REPORT_ITEM_GROUPBY_GROUPBY, group_by->get_field_group_by()->get_name());
     set_node_attribute_value(child, GLOM_ATTRIBUTE_REPORT_ITEM_GROUPBY_SORTBY, group_by->get_field_sort_by()->get_name());
+
+    if(!group_by->m_group_secondary_fields.m_map_items.empty())
+    {
+      xmlpp::Element* secondary_fields = child->add_child(GLOM_NODE_DATA_LAYOUT_GROUP_SECONDARYFIELDS);
+      save_before_layout_group(secondary_fields, group_by->m_group_secondary_fields);
+    }
   }
   else
   {
-    child = node->add_child(GLOM_NODE_DATA_LAYOUT_GROUP);
+    const LayoutItem_Summary* summary = dynamic_cast<const LayoutItem_Summary*>(&group);
+    if(summary) //If it is a GroupBy report part.
+    {
+      child = node->add_child(GLOM_NODE_DATA_LAYOUT_ITEM_SUMMARY);
+      //TODO: summary_type.
+    }
+    else
+    {
+      child = node->add_child(GLOM_NODE_DATA_LAYOUT_GROUP);
+    }
   }
 
   child->set_attribute(GLOM_ATTRIBUTE_NAME, group.get_name());
@@ -1320,59 +1454,31 @@ void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutG
     }
     else
     {
-      const LayoutItem_Field* field = dynamic_cast<const LayoutItem_Field*>(item);
-      if(field) //If it is a field
+      const LayoutItem_FieldSummary* fieldsummary = dynamic_cast<const LayoutItem_FieldSummary*>(item);
+      if(fieldsummary) //If it is a summaryfield
       {
-        xmlpp::Element* nodeItem = child->add_child(GLOM_NODE_DATA_LAYOUT_ITEM);
-        nodeItem->set_attribute(GLOM_ATTRIBUTE_NAME, item->get_name());
-        nodeItem->set_attribute(GLOM_ATTRIBUTE_RELATIONSHIP_NAME, field->get_relationship_name());
-        set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_EDITABLE, item->get_editable());
-
-        //Numeric format:
-        set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_FORMAT_THOUSANDS_SEPARATOR,  field->m_numeric_format.m_use_thousands_separator);
-        set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES_RESTRICTED, field->m_numeric_format.m_decimal_places_restricted);
-        set_node_attribute_value_as_decimal(nodeItem, GLOM_ATTRIBUTE_FORMAT_DECIMAL_PLACES, field->m_numeric_format.m_decimal_places);
-        set_node_attribute_value(nodeItem, GLOM_ATTRIBUTE_FORMAT_CURRENCY_SYMBOL, field->m_numeric_format.m_currency_symbol);
-
-        set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_CHOICES_RESTRICTED, field->get_choices_restricted());
-        set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_CHOICES_CUSTOM, field->get_has_custom_choices());
-
-        //Text formatting:
-        set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_FORMAT_TEXT_MULTILINE,  field->get_text_format_multiline());
-
-        //Choices:
-        if(field->get_has_custom_choices())
-        {
-          xmlpp::Element* child = nodeItem->add_child(GLOM_ATTRIBUTE_CHOICES_CUSTOM_LIST);
-
-          const LayoutItem_Field::type_list_values list_values = field->get_choices_custom();
-          for(LayoutItem_Field::type_list_values::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
-          {
-            xmlpp::Element* childChoice = child->add_child(GLOM_NODE_CUSTOM_CHOICE);
-            set_node_attribute_value_as_value(childChoice, GLOM_ATTRIBUTE_VALUE, *iter, field->m_field.get_glom_type());
-          }
-        }
-
-        set_node_attribute_value_as_bool(nodeItem, GLOM_ATTRIBUTE_CHOICES_RELATED, field->get_has_related_choices() );
-
-        Glib::ustring choice_relationship, choice_field, choice_second;
-        field->get_choices(choice_relationship, choice_field, choice_second);
-        set_node_attribute_value(nodeItem, GLOM_ATTRIBUTE_CHOICES_RELATED_RELATIONSHIP, choice_relationship);
-        set_node_attribute_value(nodeItem, GLOM_ATTRIBUTE_CHOICES_RELATED_FIELD, choice_field);
-        set_node_attribute_value(nodeItem, GLOM_ATTRIBUTE_CHOICES_RELATED_SECOND, choice_second);
-
-
-        set_node_attribute_value_as_decimal(nodeItem, GLOM_ATTRIBUTE_SEQUENCE, item->m_sequence);
+        xmlpp::Element* nodeItem = child->add_child(GLOM_NODE_DATA_LAYOUT_ITEM_FIELDSUMMARY);
+        save_before_layout_item_field(nodeItem, *fieldsummary);
+        set_node_attribute_value(nodeItem, GLOM_ATTRIBUTE_LAYOUT_ITEM_FIELDSUMMARY_SUMMARYTYPE, fieldsummary->get_summary_type_sql()); //The SQL name is as good as anything as an identifier for the summary function.
       }
       else
       {
-        const LayoutItem_Portal* portal = dynamic_cast<const LayoutItem_Portal*>(item);
-        if(portal) //If it is a portal
+        const LayoutItem_Field* field = dynamic_cast<const LayoutItem_Field*>(item);
+        if(field) //If it is a field
         {
-          xmlpp::Element* nodeItem = child->add_child(GLOM_NODE_DATA_LAYOUT_PORTAL);
-          nodeItem->set_attribute(GLOM_ATTRIBUTE_RELATIONSHIP_NAME, portal->get_relationship());
+          xmlpp::Element* nodeItem = child->add_child(GLOM_NODE_DATA_LAYOUT_ITEM);
+          save_before_layout_item_field(nodeItem, *field);
+        }
+        else
+        {
+          const LayoutItem_Portal* portal = dynamic_cast<const LayoutItem_Portal*>(item);
+          if(portal) //If it is a portal
+          {
+            xmlpp::Element* nodeItem = child->add_child(GLOM_NODE_DATA_LAYOUT_PORTAL);
+            nodeItem->set_attribute(GLOM_ATTRIBUTE_RELATIONSHIP_NAME, portal->get_relationship());
 
-          set_node_attribute_value_as_decimal(nodeItem, GLOM_ATTRIBUTE_SEQUENCE, item->m_sequence);
+            set_node_attribute_value_as_decimal(nodeItem, GLOM_ATTRIBUTE_SEQUENCE, item->m_sequence);
+          }
         }
       }
 
@@ -1451,6 +1557,10 @@ bool Document_Glom::save_before()
             set_node_attribute_value(elemFieldLookup, GLOM_ATTRIBUTE_RELATIONSHIP_NAME, field.get_lookup_relationship());
             set_node_attribute_value(elemFieldLookup, GLOM_ATTRIBUTE_FIELD, field.get_lookup_field());
           }
+
+          //Default Formatting:
+          xmlpp::Element* elementFormat = elemField->add_child(GLOM_NODE_FORMAT);
+          save_before_layout_item_field_formatting(elementFormat, field.m_default_formatting, field);
         }
 
         //Relationships:
@@ -1614,8 +1724,19 @@ void Document_Glom::remove_group(const Glib::ustring& group_name)
   }
 }
 
+void Document_Glom::update_cached_relationships(FieldFormatting& formatting, const Glib::ustring& table_name)
+{
+  if(formatting.m_choices_related_relationship.get_name_not_empty())
+  {
+    get_relationship(table_name, formatting.m_choices_related_relationship.get_name(), formatting.m_choices_related_relationship);
+  }
+}
+
 void Document_Glom::update_cached_relationships(LayoutGroup& group, const Glib::ustring& table_name)
 {
+  const bool old_block_modified_set = m_block_modified_set;
+  m_block_modified_set = true; //Don't let any of this cause a document save. It's just a cache.
+
   //Find any LayoutItem_Fields, and fill in their full relationship details:
   for(LayoutGroup::type_map_items::iterator iter = group.m_map_items.begin(); iter != group.m_map_items.end(); ++iter)
   {
@@ -1629,9 +1750,9 @@ void Document_Glom::update_cached_relationships(LayoutGroup& group, const Glib::
          get_relationship(table_name, pField->m_relationship.get_name(), pField->m_relationship);
        }
 
-       if(pField->m_choices_related_relationship.get_name_not_empty())
+       if(pField->m_formatting.m_choices_related_relationship.get_name_not_empty())
        {
-         get_relationship(table_name, pField->m_choices_related_relationship.get_name(), pField->m_choices_related_relationship);
+         get_relationship(table_name, pField->m_formatting.m_choices_related_relationship.get_name(), pField->m_formatting.m_choices_related_relationship);
        }
     }
     else
@@ -1655,12 +1776,24 @@ void Document_Glom::update_cached_relationships(LayoutGroup& group, const Glib::
       }
       else
       {
-        LayoutGroup* pGroup = dynamic_cast<LayoutGroup*>(pItem);
-        if(pGroup)
-          update_cached_relationships(*pGroup, table_name); //recurse:
+        LayoutItem_GroupBy* pGroupBy = dynamic_cast<LayoutItem_GroupBy*>(pItem);
+        if(pGroupBy)
+        {
+          update_cached_relationships(*pGroupBy, table_name); //recurse:
+
+          update_cached_relationships(pGroupBy->m_group_secondary_fields, table_name); //recuse.
+        }
+        else
+        {
+          LayoutGroup* pGroup = dynamic_cast<LayoutGroup*>(pItem);
+          if(pGroup)
+            update_cached_relationships(*pGroup, table_name); //recurse:
+        }
       }
     }
   }
+
+  m_block_modified_set = old_block_modified_set;
 }
 
 void Document_Glom::update_cached_relationships()
@@ -1668,11 +1801,18 @@ void Document_Glom::update_cached_relationships()
   if(m_block_cache_update)
     return;
 
-  //Update all the groups (groups are in layouts, in tables)
+
+  //Update all the formatting and groups (groups are in layouts, in tables)
   for(type_tables::iterator iterTable = m_tables.begin(); iterTable != m_tables.end(); ++iterTable)
   {
     DocumentTableInfo& tableInfo = iterTable->second;
 
+    //Fields (formatting):
+    for(type_vecFields::iterator iterField = tableInfo.m_fields.begin(); iterField != tableInfo.m_fields.end(); ++iterField)
+    {
+       update_cached_relationships(iterField->m_default_formatting, tableInfo.m_info.get_name());
+    }
+
     //Layouts:
     for(DocumentTableInfo::type_layouts::iterator iterLayout = tableInfo.m_layouts.begin(); iterLayout != tableInfo.m_layouts.end(); ++iterLayout)
     {
