@@ -28,6 +28,7 @@
 #include "data_structure/glomconversions.h"
 #include "data_structure/layout/report_parts/layoutitem_summary.h"
 #include "data_structure/layout/report_parts/layoutitem_fieldsummary.h"
+#include "relationships_overview/dialog_relationships_overview.h"
 #include <glibmm/i18n.h>
 
 Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
@@ -76,6 +77,9 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
 
     refXml->get_widget_derived("box_navigation_tables", m_pBox_Tables);
     m_pDialog_Tables = new Dialog_Glom(m_pBox_Tables);
+    
+    //Respond to window close:
+    m_pDialog_Tables->signal_hide().connect(sigc::mem_fun(*this, &Frame_Glom::on_dialog_tables_hide));
   }
   catch(const Gnome::Glade::XmlError& ex)
   {
@@ -769,6 +773,32 @@ void Frame_Glom::on_menu_developer_fields()
   }
 }
 
+void Frame_Glom::on_menu_developer_relationships_overview()
+{
+  Dialog_RelationshipsOverview* dialog = 0;
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_relationships_overview");
+    refXml->get_widget_derived("dialog_relationships_overview", dialog);
+    if(dialog)
+    {
+      dialog->set_transient_for(*(get_app_window()));
+      add_view(dialog);
+      dialog->load_from_document();
+
+      dialog->run();
+
+      remove_view(dialog);
+      delete dialog;
+    }
+  }
+
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+}
+
 void Frame_Glom::on_menu_developer_relationships()
 {
   //Check that there is a table to show:
@@ -1043,3 +1073,18 @@ void Frame_Glom::on_dialog_reports_hide()
     pApp->fill_menu_reports(m_strTableName);
 }
 
+void Frame_Glom::on_dialog_tables_hide()
+{
+  //If tables could have been added or removed, update the tables menu:
+  Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+  if(document)
+  {
+    if(document->get_userlevel() == AppState::USERLEVEL_DEVELOPER)
+    {
+      App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
+      if(pApp)
+        pApp->fill_menu_tables();
+    }
+  }
+}
+
