@@ -75,6 +75,7 @@ public:
 
   virtual void on_menu_developer_database_preferences();
   virtual void on_menu_developer_fields();
+  virtual void on_menu_developer_relationships_overview();
   virtual void on_menu_developer_relationships();
   virtual void on_menu_developer_users();
   virtual void on_menu_developer_layout();
@@ -83,6 +84,7 @@ public:
   void on_developer_dialog_hide();
   void on_dialog_layout_report_hide();
   void on_dialog_reports_hide();
+  void on_dialog_tables_hide();
 
   virtual void set_document(Document_Glom* pDocument); //View override
   virtual void load_from_document(); //View override
